from __future__ import annotations

from typing import Any

def serialize(obj: Any, *args: Any, **kwargs: Any) -> Any: ...
def deserialize(data: Any, *args: Any, **kwargs: Any) -> Any: ...
def reconnect_all(obj: Any) -> Any: ...

class SerializationError(Exception): ...
class DeserializationError(Exception): ...

__all__: list[str]
